@@ -112,24 +112,24 @@ typedef enum e_access {
 
 /* Properties of one instruction in PAGE1 (without prefix) */
 typedef struct inst_page1 {
-	unsigned insn : 9;        // A value of type m680x_insn
-	unsigned handler_id1 : 6; // Type insn_hdlr_id, first instr. handler id
-	unsigned handler_id2 : 6; // Type insn_hdlr_id, second instr. handler id
+	m680x_insn insn : 9;
+	insn_hdlr_id handler_id1 : 6; /* first instruction handler id */
+	insn_hdlr_id handler_id2 : 6; /* second instruction handler id */
 } inst_page1;
 
 /* Properties of one instruction in any other PAGE X */
 typedef struct inst_pageX {
-	unsigned opcode : 8;      // The opcode byte
-	unsigned insn : 9;        // A value of type m680x_insn
-	unsigned handler_id1 : 6; // Type insn_hdlr_id, first instr. handler id
-	unsigned handler_id2 : 6; // Type insn_hdlr_id, second instr. handler id
+	unsigned opcode : 8;
+	m680x_insn insn : 9;
+	insn_hdlr_id handler_id1 : 6; /* first instruction handler id */
+	insn_hdlr_id handler_id2 : 6; /* second instruction handler id */
 } inst_pageX;
 
 typedef struct insn_props {
 	unsigned group : 4;
-	unsigned access_mode : 5; // A value of type e_access_mode
-	unsigned reg0 : 5;        // A value of type m680x_reg
-	unsigned reg1 : 5;        // A value of type m680x_reg
+	e_access_mode access_mode : 5;
+	m680x_reg reg0 : 5;
+	m680x_reg reg1 : 5;
 	bool cc_modified : 1;
 	bool update_reg_access : 1;
 } insn_props;
@@ -619,20 +619,12 @@ typedef struct insn_desc {
 	uint16_t insn_size;
 } insn_desc;
 
-// If successfull return the additional byte size needed for M6809
-// indexed addressing mode (including the indexed addressing post_byte).
-// On error return -1.
-static int get_indexed09_post_byte_size(const m680x_info *info,
-					uint16_t address)
+static bool is_indexed09_post_byte_valid(const m680x_info *info,
+	uint16_t *address, uint8_t post_byte, insn_desc *insn_description)
 {
 	uint8_t ir = 0;
-	uint8_t post_byte;
+	bool retval;
 
-	// Read the indexed addressing post byte.
-	if (!read_byte(info, &post_byte, address))
-		return -1;
-
-	// Depending on the indexed addressing mode more bytes have to be read.
 	switch (post_byte & 0x9F) {
 	case 0x87:
 	case 0x8A:
@@ -643,71 +635,64 @@ static int get_indexed09_post_byte_size(const m680x_info *info,
 	case 0x97:
 	case 0x9A:
 	case 0x9E:
-		return -1; // illegal indexed post bytes
+		return false; // illegal indexed post bytes
 
 	case 0x88: // n8,R
 	case 0x8C: // n8,PCR
 	case 0x98: // [n8,R]
 	case 0x9C: // [n8,PCR]
-		if (!read_byte(info, &ir, address + 1))
-			return -1;
-		return 2;
+		insn_description->insn_size++;
+		return read_byte(info, &ir, (*address)++);
 
 	case 0x89: // n16,R
 	case 0x8D: // n16,PCR
 	case 0x99: // [n16,R]
 	case 0x9D: // [n16,PCR]
-		if (!read_byte(info, &ir, address + 2))
-			return -1;
-		return 3;
+		insn_description->insn_size += 2;
+		retval = read_byte(info, &ir, *address + 1);
+		*address += 2;
+		return retval;
 
 	case 0x9F: // [n]
-		if ((post_byte & 0x60) != 0 ||
-			!read_byte(info, &ir, address + 2))
-			return -1;
-		return  3;
+		insn_description->insn_size += 2;
+		retval = (post_byte & 0x60) == 0 &&
+			read_byte(info, &ir, *address + 1);
+		*address += 2;
+		return retval;
 	}
 
-	// Any other indexed post byte is valid and
+	return true; // Any other indexed post byte is valid and
 	// no additional bytes have to be read.
-	return 1;
 }
 
-// If successfull return the additional byte size needed for CPU12
-// indexed addressing mode (including the indexed addressing post_byte).
-// On error return -1.
-static int get_indexed12_post_byte_size(const m680x_info *info,
-					uint16_t address, bool is_subset)
+static bool is_indexed12_post_byte_valid(const m680x_info *info,
+	uint16_t *address, uint8_t post_byte, insn_desc *insn_description,
+	bool is_subset)
 {
 	uint8_t ir;
-	uint8_t post_byte;
+	bool result;
 
-	// Read the indexed addressing post byte.
-	if (!read_byte(info, &post_byte, address))
-		return -1;
-
-	// Depending on the indexed addressing mode more bytes have to be read.
 	if (!(post_byte & 0x20)) // n5,R
-		return 1;
+		return true;
 
 	switch (post_byte & 0xe7) {
 	case 0xe0:
 	case 0xe1: // n9,R
 		if (is_subset)
-			return -1;
+			return false;
 
-		if (!read_byte(info, &ir, address))
-			return -1;
-		return 2;
+		insn_description->insn_size++;
+		return read_byte(info, &ir, (*address)++);
 
 	case 0xe2: // n16,R
 	case 0xe3: // [n16,R]
 		if (is_subset)
-			return -1;
+			return false;
 
-		if (!read_byte(info, &ir, address + 1))
-			return -1;
-		return 3;
+		insn_description->insn_size += 2;
+		result = read_byte(info, &ir, *address + 1);
+		*address += 2;
+		return result;
 
 	case 0xe4: // A,R
 	case 0xe5: // B,R
@@ -717,7 +702,7 @@ static int get_indexed12_post_byte_size(const m680x_info *info,
 		break;
 	}
 
-	return 1;
+	return true;
 }
 
 // Check for M6809/HD6309 TFR/EXG instruction for valid register
@@ -742,57 +727,20 @@ static bool is_tfm_reg_valid(const m680x_info *info, uint8_t reg_nibble)
 	return reg_nibble <= 4;
 }
 
-// If successfull return the additional byte size needed for CPU12
-// loop instructions DBEQ/DBNE/IBEQ/IBNE/TBEQ/TBNE (including the post byte).
-// On error return -1.
-static int get_loop_post_byte_size(const m680x_info *info, uint16_t address)
+static bool is_loop_post_byte_valid(const m680x_info *info, uint8_t post_byte)
 {
-	uint8_t post_byte;
-	uint8_t rr;
-
-	if (!read_byte(info, &post_byte, address))
-		return -1;
-
 	// According to documentation bit 3 is don't care and not checked here.
-	if ((post_byte >= 0xc0) ||
-		((post_byte & 0x07) == 2) || ((post_byte & 0x07) == 3))
-		return -1;
+	if (post_byte >= 0xc0)
+		return false;
 
-	if (!read_byte(info, &rr, address + 1))
-		return -1;
-
-	return 2;
-}
-
-// If successfull return the additional byte size needed for HD6309
-// bit move instructions BAND/BEOR/BIAND/BIEOR/BIOR/BOR/LDBT/STBT
-// (including the post byte).
-// On error return -1.
-static int get_bitmv_post_byte_size(const m680x_info *info, uint16_t address)
-{
-	uint8_t post_byte;
-	uint8_t rr;
-
-	if (!read_byte(info, &post_byte, address))
-		return -1;
-
-	if ((post_byte & 0xc0) == 0xc0)
-		return -1; // Invalid register specified
-	else {
-		if (!read_byte(info, &rr, address + 1))
-			return -1;
-	}
-
-	return 2;
+	return ((post_byte & 0x07) != 2 && ((post_byte & 0x07) != 3));
 }
 
 static bool is_sufficient_code_size(const m680x_info *info, uint16_t address,
 	insn_desc *insn_description)
 {
 	int i;
-	bool retval = true;
-	uint16_t size = 0;
-	int sz;
+	bool retval;
 
 	for (i = 0; i < 2; i++) {
 		uint8_t ir = 0;
@@ -801,8 +749,9 @@ static bool is_sufficient_code_size(const m680x_info *info, uint16_t address,
 		switch (insn_description->hid[i]) {
 
 		case imm32_hid:
-			if ((retval = read_byte(info, &ir, address + size + 3)))
-				size += 4;
+			insn_description->insn_size += 4;
+			retval = read_byte(info, &ir, address + 3);
+			address += 4;
 			break;
 
 		case ext_hid:
@@ -812,8 +761,9 @@ static bool is_sufficient_code_size(const m680x_info *info, uint16_t address,
 		case opidxdr_hid:
 		case idxX16_hid:
 		case idxS16_hid:
-			if ((retval = read_byte(info, &ir, address + size + 1)))
-				size += 2;
+			insn_description->insn_size += 2;
+			retval = read_byte(info, &ir, address + 1);
+			address += 2;
 			break;
 
 		case rel8_hid:
@@ -825,8 +775,8 @@ static bool is_sufficient_code_size(const m680x_info *info, uint16_t address,
 		case idxY_hid:
 		case idxS_hid:
 		case index_hid:
-			if ((retval = read_byte(info, &ir, address + size)))
-				size++;
+			insn_description->insn_size += 1;
+			retval = read_byte(info, &ir, address++);
 			break;
 
 		case illgl_hid:
@@ -838,11 +788,14 @@ static bool is_sufficient_code_size(const m680x_info *info, uint16_t address,
 			break;
 
 		case idx09_hid:
-			sz = get_indexed09_post_byte_size(info, address + size);
-			if (sz >= 0)
-				size += sz;
-			else
+			insn_description->insn_size += 1;
+
+			if (!read_byte(info, &ir, address++))
 				retval = false;
+			else
+				retval = is_indexed09_post_byte_valid(info,
+						&address, ir, insn_description);
+
 			break;
 
 		case idx12s_hid:
@@ -851,80 +804,108 @@ static bool is_sufficient_code_size(const m680x_info *info, uint16_t address,
 		// intentionally fall through
 
 		case idx12_hid:
-			sz = get_indexed12_post_byte_size(info,
-					address + size, is_subset);
-			if (sz >= 0)
-				size += sz;
-			else
+			insn_description->insn_size += 1;
+
+			if (!read_byte(info, &ir, address++))
 				retval = false;
+			else
+				retval = is_indexed12_post_byte_valid(info,
+						&address, ir, insn_description,
+						is_subset);
+
 			break;
 
 		case exti12x_hid:
 		case imm16i12x_hid:
-			sz = get_indexed12_post_byte_size(info,
-					address + size, false);
-			if (sz >= 0) {
-				size += sz;
-				if ((retval = read_byte(info, &ir,
-						address + size + 1)))
-					size += 2;
-			} else
+			insn_description->insn_size += 1;
+
+			if (!read_byte(info, &ir, address++))
 				retval = false;
+			else if (!is_indexed12_post_byte_valid(info, &address,
+					ir, insn_description, false))
+				retval = false;
+			else {
+				insn_description->insn_size += 2;
+				retval = read_byte(info, &ir, address + 1);
+				address += 2;
+			}
+
 			break;
 
 		case imm8i12x_hid:
-			sz = get_indexed12_post_byte_size(info,
-					address + size, false);
-			if (sz >= 0) {
-				size += sz;
-				if ((retval = read_byte(info, &ir,
-						address + size)))
-					size++;
-			} else
+			insn_description->insn_size += 1;
+
+			if (!read_byte(info, &ir, address++))
 				retval = false;
+			else if (!is_indexed12_post_byte_valid(info, &address,
+					ir, insn_description, false))
+				retval = false;
+			else {
+				insn_description->insn_size += 1;
+				retval = read_byte(info, &ir, address++);
+			}
+
 			break;
 
 		case tfm_hid:
-			if ((retval = read_byte(info, &ir, address + size))) {
-				size++;
+			insn_description->insn_size += 1;
+
+			if (!read_byte(info, &ir, address++))
+				retval = false;
+			else
 				retval = is_tfm_reg_valid(info, (ir >> 4) & 0x0F) &&
 					is_tfm_reg_valid(info, ir & 0x0F);
-			}
+
 			break;
 
 		case rr09_hid:
-			if ((retval = read_byte(info, &ir, address + size))) {
-				size++;
+			insn_description->insn_size += 1;
+
+			if (!read_byte(info, &ir, address++))
+				retval = false;
+			else
 				retval = is_tfr09_reg_valid(info, (ir >> 4) & 0x0F) &&
 					is_tfr09_reg_valid(info, ir & 0x0F);
-			}
+
 			break;
 
 		case rr12_hid:
-			if ((retval = read_byte(info, &ir, address + size))) {
-				size++;
+			insn_description->insn_size += 1;
+
+			if (!read_byte(info, &ir, address++))
+				retval = false;
+			else
 				retval = is_exg_tfr12_post_byte_valid(info, ir);
-			}
+
 			break;
 
 		case bitmv_hid:
-			sz = get_bitmv_post_byte_size(info, address + size);
-			if (sz >= 0)
-				size += sz;
-			else
+			insn_description->insn_size += 2;
+
+			if (!read_byte(info, &ir, address++))
 				retval = false;
+			else if ((ir & 0xc0) == 0xc0)
+				retval = false; // Invalid register specified
+			else
+				retval = read_byte(info, &ir, address++);
+
 			break;
 
 		case loop_hid:
-			sz = get_loop_post_byte_size(info, address + size);
-			if (sz >= 0)
-				size += sz;
-			else
+			insn_description->insn_size += 2;
+
+			if (!read_byte(info, &ir, address++))
 				retval = false;
+			else if (!is_loop_post_byte_valid(info, ir))
+				retval = false;
+			else
+				retval = read_byte(info, &ir, address++);
+
 			break;
 
 		default:
-			CS_ASSERT(0 && "Unexpected instruction handler id");
+			fprintf(stderr, "Internal error: Unexpected instruction "
+				"handler id %d\n", insn_description->hid[i]);
 			retval = false;
 			break;
 		}
@@ -933,8 +914,6 @@ static bool is_sufficient_code_size(const m680x_info *info, uint16_t address,
 			return false;
 	}
 
-	insn_description->insn_size += size;
-
 	return retval;
 }
 
@@ -1090,7 +1069,7 @@ static void reg_bits_hdlr(MCInst *MI, m680x_info *info, uint16_t *address)
 	cs_m680x_op *op0 = &info->m680x.operands[0];
 	uint8_t reg_bits = 0;
 	uint16_t bit_index;
-	const m680x_reg *reg_to_reg_ids = NULL;
+	const m680x_reg *reg_to_reg_ids;
 
 	read_byte(info, &reg_bits, (*address)++);
 
@@ -1104,8 +1083,9 @@ static void reg_bits_hdlr(MCInst *MI, m680x_info *info, uint16_t *address)
 		break;
 
 	default:
-		CS_ASSERT(0 && "Unexpected operand0 register");
-		break;
+		fprintf(stderr, "Internal error: Unexpected operand0 register "
+			"%d\n", op0->reg);
+		abort();
 	}
 
 	if ((info->insn == M680X_INS_PULU ||
@@ -1549,7 +1529,8 @@ static void immediate_hdlr(MCInst *MI, m680x_info *info, uint16_t *address)
 
 	default:
 		op->imm = 0;
-		CS_ASSERT(0 && "Unexpected immediate byte size");
+		fprintf(stderr, "Internal error: Unexpected immediate byte "
+			"size %d.\n", op->size);
 	}
 
 	*address += op->size;
@@ -1728,7 +1709,7 @@ static void ext_idx12_x_hdlr(MCInst *MI, m680x_info *info, uint16_t *address)
 	indexed12_hdlr(MI, info, address);
 	read_word(info, &imm16, *address);
 	op0->type = M680X_OP_EXTENDED;
-	op0->ext.address = (int16_t)imm16;
+	op0->imm = (int16_t)imm16;
 	set_operand_size(info, op0, 1);
 }
 
@@ -1754,6 +1735,8 @@ static void loop_hdlr(MCInst *MI, m680x_info *info, uint16_t *address)
 	info->insn = index_to_insn_id[(post_byte >> 5) & 0x07];
 
 	if (info->insn == M680X_INS_ILLGL) {
+		fprintf(stderr, "Internal error: Unexpected post byte "
+			"in loop instruction %02X.\n", post_byte);
 		illegal_hdlr(MI, info, address);
 	};
 
@@ -2131,6 +2114,8 @@ static bool m680x_setup_internals(m680x_info *info, e_cpu_type cpu_type,
 	const uint8_t *code, uint16_t code_len)
 {
 	if (cpu_type == M680X_CPU_TYPE_INVALID) {
+		fprintf(stderr, "M680X_CPU_TYPE_%s is not suppported\n",
+			s_cpu_type[cpu_type]);
 		return false;
 	}
 
@@ -2208,74 +2193,85 @@ bool M680X_getInstruction(csh ud, const uint8_t *code, size_t code_len,
 cs_err M680X_disassembler_init(cs_struct *ud)
 {
 	if (M680X_REG_ENDING != ARR_SIZE(g_m6800_reg_byte_size)) {
-		CS_ASSERT(M680X_REG_ENDING == ARR_SIZE(g_m6800_reg_byte_size));
+		fprintf(stderr, "Internal error: Size mismatch in enum "
+			"m680x_reg and g_m6800_reg_byte_size\n");
 
 		return CS_ERR_MODE;
 	}
 
 	if (M680X_REG_ENDING != ARR_SIZE(g_m6801_reg_byte_size)) {
-		CS_ASSERT(M680X_REG_ENDING == ARR_SIZE(g_m6801_reg_byte_size));
+		fprintf(stderr, "Internal error: Size mismatch in enum "
+			"m680x_reg and g_m6801_reg_byte_size\n");
 
 		return CS_ERR_MODE;
 	}
 
 	if (M680X_REG_ENDING != ARR_SIZE(g_m6805_reg_byte_size)) {
-		CS_ASSERT(M680X_REG_ENDING == ARR_SIZE(g_m6805_reg_byte_size));
+		fprintf(stderr, "Internal error: Size mismatch in enum "
+			"m680x_reg and g_m6805_reg_byte_size\n");
 
 		return CS_ERR_MODE;
 	}
 
 	if (M680X_REG_ENDING != ARR_SIZE(g_m6808_reg_byte_size)) {
-		CS_ASSERT(M680X_REG_ENDING == ARR_SIZE(g_m6808_reg_byte_size));
+		fprintf(stderr, "Internal error: Size mismatch in enum "
+			"m680x_reg and g_m6808_reg_byte_size\n");
 
 		return CS_ERR_MODE;
 	}
 
 	if (M680X_REG_ENDING != ARR_SIZE(g_m6811_reg_byte_size)) {
-		CS_ASSERT(M680X_REG_ENDING == ARR_SIZE(g_m6811_reg_byte_size));
+		fprintf(stderr, "Internal error: Size mismatch in enum "
+			"m680x_reg and g_m6811_reg_byte_size\n");
 
 		return CS_ERR_MODE;
 	}
 
 	if (M680X_REG_ENDING != ARR_SIZE(g_cpu12_reg_byte_size)) {
-		CS_ASSERT(M680X_REG_ENDING == ARR_SIZE(g_cpu12_reg_byte_size));
+		fprintf(stderr, "Internal error: Size mismatch in enum "
+			"m680x_reg and g_cpu12_reg_byte_size\n");
 
 		return CS_ERR_MODE;
 	}
 
 	if (M680X_REG_ENDING != ARR_SIZE(g_m6809_reg_byte_size)) {
-		CS_ASSERT(M680X_REG_ENDING == ARR_SIZE(g_m6809_reg_byte_size));
+		fprintf(stderr, "Internal error: Size mismatch in enum "
+			"m680x_reg and g_m6809_reg_byte_size\n");
 
 		return CS_ERR_MODE;
 	}
 
 	if (M680X_INS_ENDING != ARR_SIZE(g_insn_props)) {
-		CS_ASSERT(M680X_INS_ENDING == ARR_SIZE(g_insn_props));
+		fprintf(stderr, "Internal error: Size mismatch in enum "
+			"m680x_insn and g_insn_props\n");
 
 		return CS_ERR_MODE;
 	}
 
 	if (M680X_CPU_TYPE_ENDING != ARR_SIZE(s_cpu_type)) {
-		CS_ASSERT(M680X_CPU_TYPE_ENDING == ARR_SIZE(s_cpu_type));
+		fprintf(stderr, "Internal error: Size mismatch in enum "
+			"e_cpu_type and s_cpu_type\n");
 
 		return CS_ERR_MODE;
 	}
 
 	if (M680X_CPU_TYPE_ENDING != ARR_SIZE(g_cpu_tables)) {
-		CS_ASSERT(M680X_CPU_TYPE_ENDING == ARR_SIZE(g_cpu_tables));
+		fprintf(stderr, "Internal error: Size mismatch in enum "
+			"e_cpu_type and g_cpu_tables\n");
 
 		return CS_ERR_MODE;
 	}
 
 	if (HANDLER_ID_ENDING != ARR_SIZE(g_insn_handler)) {
-		CS_ASSERT(HANDLER_ID_ENDING == ARR_SIZE(g_insn_handler));
+		fprintf(stderr, "Internal error: Size mismatch in enum "
+			"insn_hdlr_id and g_insn_handler\n");
 
 		return CS_ERR_MODE;
 	}
 
 	if (ACCESS_MODE_ENDING !=  MATRIX_SIZE(g_access_mode_to_access)) {
-		CS_ASSERT(ACCESS_MODE_ENDING ==
-			MATRIX_SIZE(g_access_mode_to_access));
+		fprintf(stderr, "Internal error: Size mismatch in enum "
+			"e_access_mode and g_access_mode_to_access\n");
 
 		return CS_ERR_MODE;
 	}
