@@ -62,15 +62,6 @@ static void printAddress(MCInst *MI, unsigned Base, int64_t Disp, unsigned Index
 			MI->flat_insn->detail->sysz.operands[MI->flat_insn->detail->sysz.op_count].imm = Disp;
 			MI->flat_insn->detail->sysz.op_count++;
 		}
-	} else {
-		SStream_concat(O, "(%%%s)", getRegisterName(Index));
-		if (MI->csh->detail) {
-			MI->flat_insn->detail->sysz.operands[MI->flat_insn->detail->sysz.op_count].type = SYSZ_OP_MEM;
-			MI->flat_insn->detail->sysz.operands[MI->flat_insn->detail->sysz.op_count].mem.base = (uint8_t)SystemZ_map_register(Base);
-			MI->flat_insn->detail->sysz.operands[MI->flat_insn->detail->sysz.op_count].mem.index = (uint8_t)SystemZ_map_register(Index);
-			MI->flat_insn->detail->sysz.operands[MI->flat_insn->detail->sysz.op_count].mem.disp = Disp;
-			MI->flat_insn->detail->sysz.op_count++;
-		}
 	}
 }
 
@@ -254,15 +245,16 @@ static void printAccessRegOperand(MCInst *MI, int OpNum, SStream *O)
 static void printPCRelOperand(MCInst *MI, int OpNum, SStream *O)
 {
 	MCOperand *MO = MCInst_getOperand(MI, OpNum);
+	int32_t imm;
 
 	if (MCOperand_isImm(MO)) {
-		int64_t imm = MCOperand_getImm(MO);
+		imm = (int32_t)MCOperand_getImm(MO);
 
-		printInt64(O, imm);
+		printInt32(O, imm);
 
 		if (MI->csh->detail) {
 			MI->flat_insn->detail->sysz.operands[MI->flat_insn->detail->sysz.op_count].type = SYSZ_OP_IMM;
-			MI->flat_insn->detail->sysz.operands[MI->flat_insn->detail->sysz.op_count].imm = imm;
+			MI->flat_insn->detail->sysz.operands[MI->flat_insn->detail->sysz.op_count].imm = (int64_t)imm;
 			MI->flat_insn->detail->sysz.op_count++;
 		}
 	}
