@@ -197,7 +197,6 @@ static void printf32mem(MCInst *MI, unsigned OpNo, SStream *O)
 			}
 			break;
 	}
-
 	printMemReference(MI, OpNo, O);
 }
 
@@ -333,34 +332,11 @@ static void _printOperand(MCInst *MI, unsigned OpNo, SStream *O)
 	} else if (MCOperand_isImm(Op)) {
 		// Print X86 immediates as signed values.
 		int64_t imm = MCOperand_getImm(Op);
-		uint8_t encsize;
-		uint8_t opsize = X86_immediate_size(MCInst_getOpcode(MI), &encsize);
-
 		if (imm < 0) {
-			if (MI->csh->imm_unsigned) {
-				if (opsize) {
-					switch(opsize) {
-						default:
-							break;
-						case 1:
-							imm &= 0xff;
-							break;
-						case 2:
-							imm &= 0xffff;
-							break;
-						case 4:
-							imm &= 0xffffffff;
-							break;
-					}
-				}
-
-				SStream_concat(O, "$0x%"PRIx64, imm);
-			} else {
-				if (imm < -HEX_THRESHOLD)
-					SStream_concat(O, "$-0x%"PRIx64, -imm);
-				else
-					SStream_concat(O, "$-%"PRIu64, -imm);
-			}
+			if (imm < -HEX_THRESHOLD)
+				SStream_concat(O, "$-0x%"PRIx64, -imm);
+			else
+				SStream_concat(O, "$-%"PRIu64, -imm);
 		} else {
 			if (imm > HEX_THRESHOLD)
 				SStream_concat(O, "$0x%"PRIx64, imm);
@@ -583,7 +559,6 @@ static void printU8Imm(MCInst *MI, unsigned Op, SStream *O)
 	if (MI->csh->detail) {
 		MI->flat_insn->detail->x86.operands[MI->flat_insn->detail->x86.op_count].type = X86_OP_IMM;
 		MI->flat_insn->detail->x86.operands[MI->flat_insn->detail->x86.op_count].imm = val;
-		MI->flat_insn->detail->x86.operands[MI->flat_insn->detail->x86.op_count].size = 1;
 		MI->flat_insn->detail->x86.op_count++;
 	}
 }
@@ -697,32 +672,12 @@ static void printOperand(MCInst *MI, unsigned OpNo, SStream *O)
 					else
 						SStream_concat(O, "$%"PRIu64, imm);
 				} else {
-					if (MI->csh->imm_unsigned) {
-						if (opsize) {
-							switch(opsize) {
-								default:
-									break;
-								case 1:
-									imm &= 0xff;
-									break;
-								case 2:
-									imm &= 0xffff;
-									break;
-								case 4:
-									imm &= 0xffffffff;
-									break;
-							}
-						}
-
-						SStream_concat(O, "$0x%"PRIx64, imm);
-					} else {
-						if (imm == 0x8000000000000000LL)  // imm == -imm
-							SStream_concat0(O, "$0x8000000000000000");
-						else if (imm < -HEX_THRESHOLD)
-							SStream_concat(O, "$-0x%"PRIx64, -imm);
-						else
-							SStream_concat(O, "$-%"PRIu64, -imm);
-					}
+					if (imm == 0x8000000000000000LL)  // imm == -imm
+						SStream_concat0(O, "$0x8000000000000000");
+					else if (imm < -HEX_THRESHOLD)
+						SStream_concat(O, "$-0x%"PRIx64, -imm);
+					else
+						SStream_concat(O, "$-%"PRIu64, -imm);
 				}
 				break;
 
@@ -808,7 +763,6 @@ static void printMemReference(MCInst *MI, unsigned Op, SStream *O)
 	MCOperand *SegReg = MCInst_getOperand(MI, Op + X86_AddrSegmentReg);
 	uint64_t ScaleVal;
 	int segreg;
-	int64_t DispVal = 1;
 
 	if (MI->csh->detail) {
 		uint8_t access[6];
@@ -837,7 +791,7 @@ static void printMemReference(MCInst *MI, unsigned Op, SStream *O)
 	}
 
 	if (MCOperand_isImm(DispSpec)) {
-		DispVal = MCOperand_getImm(DispSpec);
+		int64_t DispVal = MCOperand_getImm(DispSpec);
 		if (MI->csh->detail)
 			MI->flat_insn->detail->x86.operands[MI->flat_insn->detail->x86.op_count].mem.disp = DispVal;
 		if (DispVal) {
@@ -855,6 +809,7 @@ static void printMemReference(MCInst *MI, unsigned Op, SStream *O)
 				}
 			}
 		} else {
+            SStream_concat0(O, "0");
 		}
 	}
 
@@ -875,9 +830,6 @@ static void printMemReference(MCInst *MI, unsigned Op, SStream *O)
 			}
 		}
 		SStream_concat0(O, ")");
-	} else {
-		if (!DispVal)
-			SStream_concat0(O, "0");
 	}
 
 	if (MI->csh->detail)
